/*
FILE: PS2PadThrottle.cpp
DESCRIPTION: Implementation file for the PS2PadInputThrottle and PS2PadThrottle classes
WRITTEN BY: Derek Nedelman (nedelman@netzero.net)
DATE: August 12, 2002
HOME PAGE: http://www.playstation2-linux.com/projects/joypadlib 
*/


//Includes---------------------------------------------------------------------
#include "PS2PadThrottle.h"


//Implementation---------------------------------------------------------------

//PS2PadInputThrottle----------------------------------------------------------
void PS2PadInputThrottle::Update(float elapsedTime, bool actuallyPressed)
{
	//'elapsedTime' is the amount of time, in seconds, that has passed since the last call
	//to this function.  'actuallyPressed' is whether or not the input being throttles is actually,
	//truly, being pressed.

	pressed = false;

	if (state == NOT_PRESSED)
	{
		//Nothing was pressed before

		if (actuallyPressed)
		{
			state = PRESSED;
			holdCount = 0;
			pressed = true;
		}
	}
	else if (state == PRESSED)
	{
		//Something was pressed before

		if (actuallyPressed)
		{
			holdCount += elapsedTime;

			if (holdCount >= threshold)
			{
				if (threshold == 0.0f)
				{
					//There's no throttle threshold, so it's another press
					state = PRESSED;
				}
				else
				{
					//Holding down on the button
					state = HOLDING;				
					holdCount -= threshold;
				}
				pressed = true;
			}
		}
		else
		{
			state = NOT_PRESSED;
			holdCount = 0;
		}			
	}
	else if (state == HOLDING)
	{
		if (actuallyPressed)
		{
			holdCount += elapsedTime;

			if (holdCount >= threshold)
			{		
				state = PRESSED;
				holdCount -= threshold;
				pressed = true;
			}			
		}
		else
		{
			state = NOT_PRESSED;
			holdCount = 0;
		}
	}
}


//PS2PadThrottle---------------------------------------------------------------
void PS2PadThrottle::Update(float elapsedTime)
{
	int i;

	if (!pad)
		return;
	
	for (i = 0; i < PS2Pad::NUM_BUTTONS; i++)
		buttonThrottles[i].Update(elapsedTime, pad->IsButtonPressed(i));

	pad->GetSticks(leftStick, rightStick);
	
	stick0ThrottleLeft.Update(elapsedTime, leftStick[0] < 0.0f);
	stick0ThrottleRight.Update(elapsedTime, leftStick[0] > 0.0f);
	stick0ThrottleUp.Update(elapsedTime, leftStick[1] < 0.0f);
	stick0ThrottleDown.Update(elapsedTime, leftStick[1] > 0.0f);

	stick1ThrottleLeft.Update(elapsedTime, rightStick[0] < 0.0f);
	stick1ThrottleRight.Update(elapsedTime, rightStick[0] > 0.0f);
	stick1ThrottleUp.Update(elapsedTime, rightStick[1] < 0.0f);
	stick1ThrottleDown.Update(elapsedTime, rightStick[1] > 0.0f);
}

void PS2PadThrottle::Reset()
{
	int i;

	for (i = 0; i < PS2Pad::NUM_BUTTONS; i++)
		buttonThrottles[i].Reset();

	stick0ThrottleLeft.Reset();
	stick0ThrottleRight.Reset();
	stick0ThrottleUp.Reset();
	stick0ThrottleDown.Reset();

	stick1ThrottleLeft.Reset();
	stick1ThrottleRight.Reset();
	stick1ThrottleUp.Reset();
	stick1ThrottleDown.Reset();
}

void PS2PadThrottle::SetStickThresholds(int stick, float left, float right, float up, float down)
{
	if (stick == 0)
	{
		stick0ThrottleLeft.SetThreshold(left);
		stick0ThrottleRight.SetThreshold(right);
		stick0ThrottleUp.SetThreshold(up);
		stick0ThrottleDown.SetThreshold(down);		
	}
	else
	{
		stick1ThrottleLeft.SetThreshold(left);
		stick1ThrottleRight.SetThreshold(right);
		stick1ThrottleUp.SetThreshold(up);
		stick1ThrottleDown.SetThreshold(down);
	}
}

void PS2PadThrottle::GetSticks(float leftStick[2], float rightStick[2]) const
{
	leftStick[0] = leftStick[1] = 0.0f;
	rightStick[0] = rightStick[1] = 0.0f;

	if (stick0ThrottleLeft.IsPressed() || stick0ThrottleRight.IsPressed())	
		leftStick[0] = this->leftStick[0];

	if (stick0ThrottleUp.IsPressed() || stick0ThrottleDown.IsPressed())	
		leftStick[1] = this->leftStick[1];
	
	if (stick1ThrottleLeft.IsPressed() || stick1ThrottleRight.IsPressed())	
		rightStick[0] = this->rightStick[0];

	if (stick1ThrottleUp.IsPressed() || stick1ThrottleDown.IsPressed())	
		rightStick[1] = this->rightStick[1];	
}

float PS2PadThrottle::GetLeftX() const
{
	if (stick0ThrottleLeft.IsPressed() || stick0ThrottleRight.IsPressed())	 
		return leftStick[0]; 
	else
		return 0.0f;
}

float PS2PadThrottle::GetLeftY() const
{
	if (stick0ThrottleUp.IsPressed() || stick0ThrottleDown.IsPressed())	 
		return leftStick[1]; 
	else
		return 0.0f;
}

float PS2PadThrottle::GetRightX() const
{
	if (stick1ThrottleLeft.IsPressed() || stick1ThrottleRight.IsPressed())	 
		return rightStick[0]; 
	else
		return 0.0f;
}

float PS2PadThrottle::GetRightY() const
{
	if (stick1ThrottleUp.IsPressed() || stick1ThrottleDown.IsPressed())	 
		return rightStick[1];	 
	else
		return 0.0f;
}

void PS2PadThrottle::SetButtonThresholds(unsigned long buttons, float threshold)
{
	int i;
	
	for (i = 0; i < PS2Pad::NUM_BUTTONS; i++)
	{
		if (buttons & 1)
			buttonThrottles[i].SetThreshold(threshold);

		buttons >>= 1;
	}
}
