/*
FILE: main_throttle.cpp
DESCRIPTION: Test file demonstrating the usage of the PS2Pad library.
             Specifically, the ability to throttle the joypad is shown.
WRITTEN BY: Derek Nedelman (nedelman@netzero.net)
DATE: August 22, 2002
NOTES: The structure of the program is a bit odd. The main logic is inside
       the UpdateFrame() function, which is called at regular intervals by a
       system timer that is set by calling setitimer(). I did things in this
       way to simulate a (currently) 60 frames-per-second frame rate. Regardless
       of frame rate, the button throttling works as it should. The frame rate can
       be varied by changing the 'framesPerSecond' global variable.
HOME PAGE: http://www.playstation2-linux.com/projects/joypadlib 
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include "PS2Pad.h"
#include "PS2PadThrottle.h"

PS2PadThrottle throttle;
bool done = false;	//Used in UpdateFrame() to exit out of the main() loop

const int framesPerSecond = 60;  //The simulated frame rate of the main loop
const float elapsedTimePerFrame = 1.0f/framesPerSecond;
const int microsecondsPerFrame = (int)(elapsedTimePerFrame * 1000000.0f);

void UpdateFrame(int theSignal)
{
	PS2PadManager::Update(elapsedTimePerFrame);
	throttle.Update(elapsedTimePerFrame);
		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_LEFT))
	{
		puts("Pressed left");
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_UP))
	{
		puts("Pressed up");
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_RIGHT))
	{
		puts("Pressed right");
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_DOWN))
	{
		puts("Pressed down");
	}
		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_L1))
	{
		puts("Pressed L1");
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_L2))
	{
		puts("Pressed L2");
	}
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_L3))
	{
		puts("Pressed L3");
	}
		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_R1))
	{
		puts("Pressed R1");
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_R2))
	{
		puts("Pressed R2");
	}
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_R3))
	{
		puts("Pressed R3");
	}
	
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_START))
	{
		puts("Pressed start");
		done = true;
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_SELECT))
	{
		puts("Pressed select");
	}
		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_TRIANGLE))
	{
		puts("Pressed triangle");
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_CIRCLE))
	{			
		puts("Pressed circle");
	}		
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_SQUARE))
	{
		puts("Pressed square");
	} 
	if (throttle.IsButtonPressed(PS2Pad::BUTTON_CROSS))
	{
		puts("Pressed cross");
	}	

	if ((throttle.GetLeftX() != 0.0f) || (throttle.GetLeftY() != 0.0f))
	{
		printf("Left stick x/y: %f %f\n", throttle.GetLeftX(), throttle.GetLeftY());
	}

	if ((throttle.GetRightX() != 0.0f) || (throttle.GetRightY() != 0.0f))
	{
		printf("Right stick x/y: %f %f\n", throttle.GetRightX(), throttle.GetRightY());
	}
}

int main(int argc, char** argv)
{
	const float delay = .1f;	//Number of seconds between button presses

	PS2PadManager::Initialize();

	throttle.SetPad(PS2PadManager::GetPad(0));
	throttle.SetButtonThresholds(0xffffffff, delay);  //0xffffffff means "all buttons"
	throttle.SetStickThresholds(0, delay, delay, delay, delay);
	throttle.SetStickThresholds(1, delay, delay, delay, delay);
	
	puts("Press anything on the joypad to see something happen.");
	printf("All buttons and sticks are on a %f second(s) delay.\n", delay);
	printf("This means you need to hold a button down for %f second(s) for consecutive presses.\n", delay);
	puts("Of course, you may also tap the buttons to make them 'fire' faster.");
	puts("Press the START button to exit.");

	signal(SIGALRM, UpdateFrame);

	itimerval timer;
	timer.it_interval.tv_sec = 0;
	timer.it_interval.tv_usec = 0;
	timer.it_value.tv_sec = 0;
	timer.it_value.tv_usec = microsecondsPerFrame;

	while (!done)
	{				
		setitimer(ITIMER_REAL, &timer, 0);
		pause();
	}

	PS2PadManager::Shutdown();

	return 0;
}
